﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Input;
using combit.Reporting;
using combit.Reporting.DataProviders;
using GalaSoft.MvvmLight.CommandWpf;
using LL28Test.Contracts;

namespace LL28Test
{
    public class MainViewModel
    {
        private ListLabel _listLabel;
        private readonly string _projektDateiPfad = Path.GetFullPath(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, @"Reports\ZinsreportEinzelpersonFlat.lst"));

        public MainViewModel()
        {

            Designer = new RelayCommand(DesignerExecute);
            Preview = new RelayCommand(PreviewExecute);
            ExcelExport = new RelayCommand(ExcelExportExecute);
        }

        public ICommand Designer { get; }
        public ICommand Preview { get; }
        public ICommand ExcelExport { get; }

        private void DesignerExecute()
        {
            ErzeugeReport();
            BuildDesign();
        }

        private void PreviewExecute()
        {
            ErzeugeReport();
            BuildPreview();
        }

        private void ExcelExportExecute()
        {
            ErzeugeReport();
            BuildExcelExport();
        }

        private void ErzeugeReport()
        {
            _listLabel = ErzeugeListAndLabelEngine();
            var reportDaten = ErzeugeTestDaten();
            var berichtsparameter = ErzeugeBerichtsparameter();
            var collection = new DataProviderCollection
            {
                new ObjectDataProvider(reportDaten) { RootTableName = "ReportDaten" },
                new ObjectDataProvider(berichtsparameter) { RootTableName = "Berichtsparameter" }
            };
            _listLabel.DataSource = collection;
            _listLabel.Variables.AddFromObject("Kopfdaten", ErzeugeKopfdaten());
        }

        private ICollection<ZinsreportEinzelpersonDto> ErzeugeTestDaten()
        {
            return new List<ZinsreportEinzelpersonDto>
            {
                new ZinsreportEinzelpersonDto
                {
                    Kalendertag = DateTime.Today,
                    Forderungen = 2000000m,
                    Saldo = 2000000m
                },
                new ZinsreportEinzelpersonDto
                {
                    Kalendertag = DateTime.Today.AddDays(1),
                    Verbindlichkeiten = 1000000,
                    Saldo = -1000000
                }
            };
        }

        private Kopfdaten ErzeugeKopfdaten()
        {
            return new Kopfdaten
            {
                Benutzer = "user",
                Mandant = "mandant",
                Titel = "TEST"
            };
        }

        private List<Berichtsparameter> ErzeugeBerichtsparameter()
        {
            return new List<Berichtsparameter>
            {
                new Berichtsparameter { Label = "Verbindlichkeiten:", Wert = "erhaltene Sicherheiten" },
                new Berichtsparameter { Label = "Forderungen:", Wert = "gestellte Sicherheiten" }
            };
        }

        private ListLabel ErzeugeListAndLabelEngine()
        {
            var listLabel = new ListLabel
            {
                LicensingInfo = "TODO", // TODO Lizenzschlüssel
                AutoProjectType = LlProject.List,
                AutoShowPrintOptions = false
            };

            listLabel.AutoProjectType = LlProject.List;
            listLabel.AutoProjectFile = _projektDateiPfad;
            listLabel.AutoShowSelectFile = false;
            listLabel.ExportOptions.Add(LlExportOption.XlsFontScalingPercentage, "100");

            return listLabel;
        }

        private void BuildDesign()
        {
            _listLabel.AutoShowSelectFile = string.IsNullOrEmpty(_projektDateiPfad) || !File.Exists(_projektDateiPfad);
            _listLabel.Design();
        }

        private void BuildPreview()
        {
            _listLabel.ExportOptions.Add(LlExportOption.ExportTarget, "PRV");
            _listLabel.ExportOptions.Add(LlExportOption.PdfExcludedFonts, string.Empty);
            _listLabel.Core.LlSetOptionString(LlOptionString.ExportsAllowedInPreview, "PRN;PDF;XLS;XPS;PICTURE_PNG;PICTURE_JPEG");

            _listLabel.Print();
        }

        private void BuildExcelExport()
        {

            _listLabel.ExportOptions.Add(LlExportOption.XlsWorksheetName, "Test");

            var target = LlExportTarget.Xlsx;

            var config = new ExportConfiguration(target, @"\Test.xlsx", _projektDateiPfad) { ShowResult = true };

            _listLabel.Export(config);
        }
    }
}
